﻿EnableExplicit

;- -- Information.
; Dieses Modul ist das Hauptfenster des Projekts. Es wird nur für das Testen des LibUsb-Moduls verwendet.

;- -- Version 01.
; Status:   In Arbeit.
; Autor:    Peter Dreisiebner
; Website:  http://peter.dreisiebner.at
; Geändert: 2017-10-13 - PureBasic v5.61 (x86).


;-
;- --DeclareModule WindowStart.
;
DeclareModule WindowStart
   EnableExplicit

   Declare.i Open()

EndDeclareModule  ; WindowStart.

;-
;- -- Include.
;

;-
;- -- Module WindowStart.
;
Module WindowStart
   EnableExplicit
   UseModule Common
   UseModule Tool
   DebugOutWindowStart
   
   ;-
   ;- -- Konstanten.
   #iWindowWidth = 800
   #iWindowHeight = 450
   #iWindowMinWidth = 150
   #iWindowMinHeight = 150
   CompilerIf #fDebugOut Or #fDebugSpeed
      #sWindowTitle = #sApplicationName + " (Debugversion)"
   CompilerElse
      #sWindowTitle = #sApplicationName
   CompilerEndIf

   #sFontName = "Courier New"
   #iFontSize = 9

   #sMenuTitleFile = "&Datei"
   #sMenuItemFileExit = "&Beenden"
   #sMenuTitleEdit = "&Bearbeiten"
   #sMenuTitleHelp = "&Hilfe"
   #sMenuItemHelpLog = "&Log..."
   #sMenuItemHelpAbout = "&Über..."
   Enumeration eMenuItemId
      #eMenuItemFileExit
      #eMenuItemHelpLog
      #eMenuItemHelpAbout
   EndEnumeration

   Enumeration
      #iListViewGetVersion
      #iListViewErrorName
      #iListViewErrorDescription
      #iListViewDevices
   EndEnumeration
   
   ;-
   ;- -- Variablen.
   Global miWindowNumber.i
   Global miMenuNumber.i
   Global miListViewGadgetNumber.i
   Global miEditorGadgetNumber.i
   Global miSplitterGadgetNumber.i
   Global msLine.s = "------------------------------------------------------"
   
   ;-
   ;- -- Declares.
   Declare Close()

   Declare LibUsbErrorDescription()
   Declare LibUsbErrorName()
   Declare LibUsbGetVersion()
   Declare LibUsbDevices()
   Declare LibUsbContext(*uContext.Integer)
   Declare LibUsbHasCapability()
   Declare LibUsbGetDeviceList(*uContext, List *DeviceList.LibUsb::uDevice(), *FreeDeviceList.Integer, *sError.String)
   Declare LibUsbDevice(*uDevice, iDevice.i)
   Declare LibUsbGetNumbers(*uDevice)
   Declare LibUsbOpen(*uDevice, *uDeviceHandle.Integer)
   Declare LibUsbDesriptor(*uDevice)
   Declare LibUsbDescriptorLanguageList(*uDeviceHandle)
   Declare LibUsbDescriptorStringAscii(*uDeviceHandle)
   Declare LibUsbDescriptorStringUnicode(*uDeviceHandle)
   Declare LibUsbSetConfiguration(*uDeviceHandle)
   Declare LibUsbGetActiveConfigDescriptor(*uDevice, *uConfigDescriptor.Integer)
   Declare LibUsbInterface(*uDeviceHandle, *uConfigDescriptor.LibUsb::uConfigDescriptor)
   Declare LibUsbEndpoint(*uInterfaceDescriptor)
   Declare LibUsbFreeConfigDescriptor(*uConfigDescriptor)
   Declare.i LibUsbClaimInterface(*uDeviceHandle, iInterface.i)
   Declare LibUsbReleaseInterface(*uDeviceHandle, iInterface.i)

   ;-

   Procedure.i EventCloseWindow()
      ; Ereignisprozedur wenn das Fenster geschlossen werden soll.
      DebugView::ProcIn()

      Close()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMinimizeWindow()
      DebugView::ProcIn()

      WindowLog::Hide()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventSizeWindow()
      ; Ereignisprozedur wenn das Fenster in der Größe geändert wurde.
      ; Die Gadgets an die neue Größe des Fensters anpassen.
      DebugView::ProcIn()
      Protected iWidth.i, iHeight.i

      iWidth = WindowWidth(miWindowNumber)
      iHeight = (WindowHeight(miWindowNumber) - MenuHeight())

      If miSplitterGadgetNumber
         ResizeGadget(miSplitterGadgetNumber, #PB_Ignore, #PB_Ignore, iWidth - 20, iHeight - 20)
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure EventRestoreWindow()
      DebugView::ProcIn()

      WindowLog::RestoreWindow()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuFileExit()
      DebugView::ProcIn()

      Close()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuHelpLog()
      DebugView::ProcIn()

      WindowLog::Show(miWindowNumber)

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuHelpAbout()
      DebugView::ProcIn()

      WindowAbout::Open(miWindowNumber)

      DebugView::ProcOut()
   EndProcedure


   Procedure EventListViewGadgetLeftClick()
      ; Ereignis wenn ein Eintrag in dem ListViewGadget ausgewählt wurde.
      DebugView::ProcIn()
      Protected iItem.i

      iItem = GetGadgetState(miListViewGadgetNumber)
      Select iItem
         Case #iListViewGetVersion
            LibUsbGetVersion()
         Case #iListViewErrorName
            LibUsbErrorName()
         Case #iListViewErrorDescription
            LibUsbErrorDescription()
         Case #iListViewDevices
            LibUsbDevices()
      EndSelect

      DebugView::ProcOut()
   EndProcedure

   ;-

   Procedure LibUsbGetVersion()
      DebugView::ProcIn()
      Protected fResult.i
      Protected sVersion.String, sDescription.String, sError.String

      ClearGadgetItems(miEditorGadgetNumber)

      If LibUsb::GetVersion(sVersion, sDescription, sError)
         AddGadgetItem(miEditorGadgetNumber, -1, "Version: " + sVersion\s)
         AddGadgetItem(miEditorGadgetNumber, -1, "Beschreibung: " + sDescription\s)
      Else
         AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure LibUsbErrorName()
      DebugView::ProcIn()
      Protected ic.i

      ClearGadgetItems(miEditorGadgetNumber)

      For ic = 0 To -13 Step -1
         AddGadgetItem(miEditorGadgetNumber, -1, "(" + Str(ic) + ") " + LibUsb::ErrorName(ic))
      Next ic
      AddGadgetItem(miEditorGadgetNumber, -1, "(" + Str(-99) + ") " + LibUsb::ErrorName(-99))

      DebugView::ProcOut()
   EndProcedure


   Procedure LibUsbErrorDescription()
      DebugView::ProcIn()
      Protected ic.i, ik.i
      Protected sLocale.s, sError.String

      ClearGadgetItems(miEditorGadgetNumber)

      For ik = 0 To 4
         Select ik
            Case 0 : sLocale = "--"
            Case 1 : sLocale = "en"
            Case 2 : sLocale = "nl"
            Case 3 : sLocale = "fr"
            Case 4 : sLocale = "ru"
         EndSelect

         AddGadgetItem(miEditorGadgetNumber, -1, "SetLocale: " + sLocale)
         If LibUsb::SetLocale(sLocale, sError)
            For ic = 0 To -13 Step -1
               AddGadgetItem(miEditorGadgetNumber, -1, LibUsb::ErrorDescription(ic))
            Next ic
            AddGadgetItem(miEditorGadgetNumber, -1, LibUsb::ErrorDescription(-99))
         Else
            AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
         EndIf
         AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
      Next ik

      LibUsb::SetLocale("en", sError)

      DebugView::ProcOut()
   EndProcedure

   ;-
   Procedure LibUsbDevices()
      DebugView::ProcIn()
      Protected fResult.i, fInitialized.i, fDeviceList.i, sError.String
      Protected *uContext
      NewList *DeviceList.LibUsb::uDevice()
      Protected *FreeDeviceList
      Protected *uDevice.LibUsb::uDevice
      Protected *uDeviceHandle
;     Protected *uDeviceDescriptor.LibUsb::uDeviceDescriptor
      Protected *uConfigDescriptor
      Protected c.i

      ; TODO - Überprüfen ob ein Speicherzugriff erlaubt ist?

      SendMessage_(GadgetID(miEditorGadgetNumber), #WM_SETREDRAW, #False, 0)

      ClearGadgetItems(miEditorGadgetNumber)
      AddGadgetItem(miEditorGadgetNumber, -1, "Die Suche nach USB-Geräten wurde gestartet.")

      ;- Init libusb.
      *uContext = #Null
      AddGadgetItem(miEditorGadgetNumber, -1, msLine)
      AddGadgetItem(miEditorGadgetNumber, -1, "Init libusb - Standard Context verwenden.")
      fInitialized = LibUsbContext(*uContext)   ; Standard-Context.
      ;AddGadgetItem(miEditorGadgetNumber, -1, "Init libusb - Eigenen Context verwenden.")
      ;fInitialized = LibUsbContext(@*uContext) ; Eigener Context.

      ;- DebugLevel setzen.
      LibUsb::SetDebug(*uContext, LibUsb::#eLIBUSB_LOG_LEVEL_DEBUG)

      ;- Fähigkeiten der libusb abfragen.
      LibUsbHasCapability()

      ;- Geräteliste ermitteln.
      If fInitialized
         fDeviceList = LibUsbGetDeviceList(*uContext, *DeviceList(), @*FreeDeviceList, sError)
      EndIf

      ;- Geräteinformationen ausgeben.
      If fDeviceList
         ForEach *DeviceList()
            c + 1

            ;- Device-Struktur.
            AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
            AddGadgetItem(miEditorGadgetNumber, -1, msLine)
            *uDevice = *DeviceList()
            LibUsbDevice(*uDevice, c)

            ;- Bus-/Portnummern und Geschwindigkeit ermitteln.
            LibUsbGetNumbers(*uDevice)

            ;- Gerät öffnen.
            LibUsbOpen(*uDevice, @*uDeviceHandle)

            ;- DeviceDescriptor ermitteln.
            LibUsbDesriptor(*uDevice)
            
            ;- Ländercodes für die String Descriptoren ermitteln.
            LibUsbDescriptorLanguageList(*uDeviceHandle)

            ;- Device Unicode String-Descriptoren ermitteln.
            LibUsbDescriptorStringUnicode(*uDeviceHandle)

            ;- Device ASCII String-Descriptoren ermitteln.
            LibUsbDescriptorStringAscii(*uDeviceHandle)

            ;- Konfiguration setzen.
            LibUsbSetConfiguration(*uDeviceHandle)

            ;- Aktive Konfiguration ermitteln.
            LibUsbGetActiveConfigDescriptor(*uDevice, @*uConfigDescriptor)

            ;- Interfaces und Endpoints der aktiven Konfiguration ermitteln.
            LibUsbInterface(*uDeviceHandle, *uConfigDescriptor)

            ;- Aktive Konfiguration freigeben.
            LibUsbFreeConfigDescriptor(*uConfigDescriptor)
            *uConfigDescriptor = #Null

            ;- Interface reservieren.
            LibUsbClaimInterface(*uDeviceHandle, 0)

            ;- Interface freigeben.
            LibUsbReleaseInterface(*uDeviceHandle, 0)

            ;- Gerät schließen.
            LibUsb::Close(*uDeviceHandle)
            *uDeviceHandle = #Null

         Next
         AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
      EndIf

      ;- Geräteliste freigeben.
      If fDeviceList
         AddGadgetItem(miEditorGadgetNumber, -1, msLine)
         AddGadgetItem(miEditorGadgetNumber, -1, "Geräteliste freigeben.")
         LibUsb::FreeDeviceList(*FreeDeviceList, #True)
         *FreeDeviceList = #Null
      EndIf

      ;- Exit libusb.
      If fInitialized
         AddGadgetItem(miEditorGadgetNumber, -1, msLine)
         AddGadgetItem(miEditorGadgetNumber, -1, "Exit libusb.")
         LibUsb::Exit(*uContext)
         *uContext = #Null
      EndIf

      SendMessage_(GadgetID(miEditorGadgetNumber), #WM_SETREDRAW, #True, 0)
      ResizeGadget(miEditorGadgetNumber, #PB_Ignore, #PB_Ignore, #PB_Ignore, #PB_Ignore)

      DebugView::ProcOut()
      ProcedureReturn
   EndProcedure


   Procedure.i LibUsbContext(*uContext.Integer)
      DebugView::ProcIn()
      Protected fReturn.i, *uContextLocal.LibUsb::uContext, sError.String

      fReturn = LibUsb::Init(*uContext, sError)
      If (Not fReturn)
         AddGadgetItem(miEditorGadgetNumber, -1, sError\s)

      ElseIf *uContext
         AddGadgetItem(miEditorGadgetNumber, -1, "  *uContext: $" + Hex(*uContext\i))
         *uContextLocal = *uContext
         With *uContextLocal
            AddGadgetItem(miEditorGadgetNumber, -1, "")
            AddGadgetItem(miEditorGadgetNumber, -1, "  iDebug.i: " + \iDebug)
            AddGadgetItem(miEditorGadgetNumber, -1, "  iDebugFixed.i: " + \iDebugFixed)
            AddGadgetItem(miEditorGadgetNumber, -1, "  iEventPipe[0]: " + \iEventPipe[0])
            AddGadgetItem(miEditorGadgetNumber, -1, "  iEventPipe[1]: " + \iEventPipe[1])
            AddGadgetItem(miEditorGadgetNumber, -1, "  uUsbDevs.uListHead\uPrev: $" + Hex(\uUsbDevs\uPrev))
            AddGadgetItem(miEditorGadgetNumber, -1, "  uUsbDevs.uListHead\uNext: $" + Hex(\uUsbDevs\uNext))
            AddGadgetItem(miEditorGadgetNumber, -1, "  iUsbDevsLock.i: " + \iUsbDevsLock)
            AddGadgetItem(miEditorGadgetNumber, -1, "  uOpenDevs.uListHead\uPrev: $" + Hex(\uOpenDevs\uPrev))
            AddGadgetItem(miEditorGadgetNumber, -1, "  uOpenDevs.uListHead\uNext: $" + Hex(\uOpenDevs\uNext))
            AddGadgetItem(miEditorGadgetNumber, -1, "  iOpenDevsLock.i: " + \iOpenDevsLock)
            AddGadgetItem(miEditorGadgetNumber, -1, "  uHotplugCbs.uListHead\uPrev: $" + Hex(\uHotplugCbs\uPrev))
            AddGadgetItem(miEditorGadgetNumber, -1, "  uHotplugCbs.uListHead\uNext: $" + Hex(\uHotplugCbs\uNext))
            AddGadgetItem(miEditorGadgetNumber, -1, "  iHotplugCbsLock.i: " + \iHotplugCbsLock)
            AddGadgetItem(miEditorGadgetNumber, -1, "  uFlyingTransfers.uListHead\uPrev: $" + Hex(\uFlyingTransfers\uPrev))
            AddGadgetItem(miEditorGadgetNumber, -1, "  uFlyingTransfers.uListHead\uNext: $" + Hex(\uFlyingTransfers\uNext))
            AddGadgetItem(miEditorGadgetNumber, -1, "  iFlyingTransfersLock.i: " + \iFlyingTransfersLock)
            AddGadgetItem(miEditorGadgetNumber, -1, "  *FdAddedCb: " + Hex(\FdAddedCb))
            AddGadgetItem(miEditorGadgetNumber, -1, "  *FdRemovedCb: " + Hex(\FdRemovedCb))
            AddGadgetItem(miEditorGadgetNumber, -1, "  *FdCbUserData: " + Hex(\FdCbUserData))
            AddGadgetItem(miEditorGadgetNumber, -1, "  iEventsLock.i: " + \iEventsLock)
            AddGadgetItem(miEditorGadgetNumber, -1, "  iEventHandlerActive.i: " + \iEventHandlerActive)
            AddGadgetItem(miEditorGadgetNumber, -1, "  iEventHandlingKey.i: " + \iEventHandlingKey)
            AddGadgetItem(miEditorGadgetNumber, -1, "  iEventWaitersLock.i: " + \iEventWaitersLock)
            AddGadgetItem(miEditorGadgetNumber, -1, "  iEventWaitersCond.i: " + \iEventWaitersCond)
            AddGadgetItem(miEditorGadgetNumber, -1, "  iEventDataLock.i: " + \iEventDataLock)
            AddGadgetItem(miEditorGadgetNumber, -1, "  iEventFlags.i: " + \iEventFlags)
            AddGadgetItem(miEditorGadgetNumber, -1, "  iDeviceClose.i: " + \iDeviceClose)
            AddGadgetItem(miEditorGadgetNumber, -1, "  uPollFds.uListHead\uPrev: $" + Hex(\uPollFds\uPrev))
            AddGadgetItem(miEditorGadgetNumber, -1, "  uPollFds.uListHead\uNext: $" + Hex(\uPollFds\uNext))
            AddGadgetItem(miEditorGadgetNumber, -1, "  *PollFds: " + Hex(\PollFds))
            AddGadgetItem(miEditorGadgetNumber, -1, "  iPollFdsCnt.i: " + \iPollFdsCnt)
            AddGadgetItem(miEditorGadgetNumber, -1, "  uHotplugMsgs.uListHead\uPrev: $" + Hex(\uHotplugMsgs\uPrev))
            AddGadgetItem(miEditorGadgetNumber, -1, "  uHotplugMsgs.uListHead\uNext: $" + Hex(\uHotplugMsgs\uNext))
            AddGadgetItem(miEditorGadgetNumber, -1, "  uCompletedTransfers.uListHead\uPrev: $" + Hex(\uCompletedTransfers\uPrev))
            AddGadgetItem(miEditorGadgetNumber, -1, "  uCompletedTransfers.uListHead\uNext: $" + Hex(\uCompletedTransfers\uNext))
            AddGadgetItem(miEditorGadgetNumber, -1, "  iTimerFd.i: " + \iTimerFd)
            AddGadgetItem(miEditorGadgetNumber, -1, "  uList.uListHead\uPrev: $" + Hex(\uList\uPrev))
            AddGadgetItem(miEditorGadgetNumber, -1, "  uList.uListHead\uNext: $" + Hex(\uList\uNext))
         EndWith
      Else
         AddGadgetItem(miEditorGadgetNumber, -1, "  *uContext: $" + Hex(*uContext))
   EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure LibUsbHasCapability()
      DebugView::ProcIn()
      Protected fResult.i

      AddGadgetItem(miEditorGadgetNumber, -1, msLine)
      AddGadgetItem(miEditorGadgetNumber, -1, "HasCapability")
      fResult = LibUsb::HasCapability(LibUsb::#eLIBUSB_CAP_HAS_CAPABILITY)
      AddGadgetItem(miEditorGadgetNumber, -1, "  LIBUSB_CAP_HAS_CAPABILITY: " + StringBool(fResult))
      fResult = LibUsb::HasCapability(LibUsb::#eLIBUSB_CAP_HAS_HOTPLUG)
      AddGadgetItem(miEditorGadgetNumber, -1, "  LIBUSB_CAP_HAS_HOTPLUG: " + StringBool(fResult))
      fResult = LibUsb::HasCapability(LibUsb::#eLIBUSB_CAP_HAS_HID_ACCESS)
      AddGadgetItem(miEditorGadgetNumber, -1, "  LIBUSB_CAP_HAS_HID_ACCESS: " + StringBool(fResult))
      fResult = LibUsb::HasCapability(LibUsb::#eLIBUSB_CAP_SUPPORTS_DETACH_KERNEL_DRIVER)
      AddGadgetItem(miEditorGadgetNumber, -1, "  LIBUSB_CAP_SUPPORTS_DETACH_KERNEL_DRIVER: " + StringBool(fResult))

      DebugView::ProcOut()
   EndProcedure


   Procedure.i LibUsbGetDeviceList(*uContext, List *DeviceList.LibUsb::uDevice(), *FreeDeviceList.Integer, *sError.String)
      DebugView::ProcIn()
      Protected fReturn.i, iDevices.i, c.i

      AddGadgetItem(miEditorGadgetNumber, -1, msLine)
      AddGadgetItem(miEditorGadgetNumber, -1, "Geräteliste ermitteln.")

      fReturn = LibUsb::GetDeviceList(*uContext, *DeviceList(), @iDevices, *FreeDeviceList, *sError)
      If (Not fReturn)
         AddGadgetItem(miEditorGadgetNumber, -1, *sError\s)
         *FreeDeviceList\i = #Null
      Else
         AddGadgetItem(miEditorGadgetNumber, -1, "  iDevices: " + iDevices)
         AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
         ForEach *DeviceList()
            c + 1
            AddGadgetItem(miEditorGadgetNumber, -1, "  Device " + Str(c) + ": $" + Hex(*DeviceList()))
         Next
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure LibUsbDevice(*uDevice.LibUsb::uDevice, iDevice.i)
      DebugView::ProcIn()

      If *uDevice
         With *uDevice
            AddGadgetItem(miEditorGadgetNumber, -1, "")
            AddGadgetItem(miEditorGadgetNumber, -1, "Device " + Str(iDevice) + ": $" + Hex(*uDevice))
            AddGadgetItem(miEditorGadgetNumber, -1, "  iLock.i: " + \iLock)
            AddGadgetItem(miEditorGadgetNumber, -1, "  iRefCnt.i: " + \iRefCnt)
            AddGadgetItem(miEditorGadgetNumber, -1, "  *uContext: $" + Hex(\uContext))
            AddGadgetItem(miEditorGadgetNumber, -1, "  iBusNumber.a: " + \iBusNumber)
            AddGadgetItem(miEditorGadgetNumber, -1, "  iPortNumber.a: " + \iPortNumber)
            AddGadgetItem(miEditorGadgetNumber, -1, "  *uParentDev: $" + Hex(\uParentDev))
            AddGadgetItem(miEditorGadgetNumber, -1, "  iDeviceAddress.a: " + \iDeviceAddress)
            AddGadgetItem(miEditorGadgetNumber, -1, "  iNumConfigurations.a: " + \iNumConfigurations)
            AddGadgetItem(miEditorGadgetNumber, -1, "  eSpeed.i: " + \eSpeed)
            AddGadgetItem(miEditorGadgetNumber, -1, "  uList.*uPrev: $" + Hex(\uList\uPrev))
            AddGadgetItem(miEditorGadgetNumber, -1, "  uList.*uNext: $" + Hex(\uList\uNext))
            AddGadgetItem(miEditorGadgetNumber, -1, "  iSessionData.l: " + \iSessionData)
            AddGadgetItem(miEditorGadgetNumber, -1, "  uDeviceDescriptor: $" + Hex(\uDeviceDescriptor))
            AddGadgetItem(miEditorGadgetNumber, -1, "  iAttached.i: " + \iAttached)
            AddGadgetItem(miEditorGadgetNumber, -1, "  iOSpriv.a[0]: " + \iOSpriv[0])
         EndWith
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure LibUsbGetNumbers(*uDevice)
      DebugView::ProcIn()
      Protected iNumber.i, sError.String

      AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
      AddGadgetItem(miEditorGadgetNumber, -1, "GetBus/PortNumber, GetDeviceSpeed")
      If *uDevice
         If LibUsb::GetBusNumber(*uDevice, @iNumber, sError)
            AddGadgetItem(miEditorGadgetNumber, -1, "  Bus: " + iNumber)
         Else
            AddGadgetItem(miEditorGadgetNumber, -1, "  Bus: " + sError\s)
         EndIf
         If LibUsb::GetPortNumber(*uDevice, @iNumber, sError)
            AddGadgetItem(miEditorGadgetNumber, -1, "  Port: " + iNumber)
         Else
            AddGadgetItem(miEditorGadgetNumber, -1, "  Port: " + sError\s)
         EndIf
         If LibUsb::GetDeviceSpeed(*uDevice, @iNumber, sError)
            AddGadgetItem(miEditorGadgetNumber, -1, "  Speed: " + iNumber)
         Else
            AddGadgetItem(miEditorGadgetNumber, -1, "  Speed: " + sError\s)
         EndIf
      Else
         AddGadgetItem(miEditorGadgetNumber, -1, "  *uDevice ist #Null")
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure LibUsbOpen(*uDevice, *uDeviceHandle.Integer)
      DebugView::ProcIn()
      Protected *uDeviceHandleLocal.LibUsb::uDeviceHandle, sError.String

      If *uDevice
         AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
         If (Not LibUsb::Open(*uDevice, *uDeviceHandle, sError))
            AddGadgetItem(miEditorGadgetNumber, -1, "Gerät öffnen: " + sError\s)
         Else
            *uDeviceHandleLocal = *uDeviceHandle
            With *uDeviceHandleLocal
               AddGadgetItem(miEditorGadgetNumber, -1, "DeviceHandle $" + Hex(*uDeviceHandle\i))
               AddGadgetItem(miEditorGadgetNumber, -1, "  iLock.i: " + \iLock)
               AddGadgetItem(miEditorGadgetNumber, -1, "  iClaimedInterfaces.l: " + \iClaimedInterfaces)
               AddGadgetItem(miEditorGadgetNumber, -1, "  uList.*uPrev: $" + Hex(\uList\uPrev))
               AddGadgetItem(miEditorGadgetNumber, -1, "  uList.*uNext: $" + Hex(\uList\uNext))
               AddGadgetItem(miEditorGadgetNumber, -1, "  *uDev.uDevice: $" + Hex(\uDev))
               AddGadgetItem(miEditorGadgetNumber, -1, "  iAutoDetachKernelDriver.i: " + \iAutoDetachKernelDriver)
               AddGadgetItem(miEditorGadgetNumber, -1, "  iOSpriv.a[0]: " + \iOSpriv[0])
            EndWith
         EndIf
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure LibUsbDesriptor(*uDevice)
      DebugView::ProcIn()
      Protected sError.String
      Protected uDeviceDescriptor.LibUsb::uDeviceDescriptor

      If *uDevice
         If (Not LibUsb::GetDeviceDescriptor(*uDevice, uDeviceDescriptor, sError))
            AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
         Else
            With uDeviceDescriptor
               AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
               AddGadgetItem(miEditorGadgetNumber, -1, "DeviceDescriptor")
               AddGadgetItem(miEditorGadgetNumber, -1, "  iLength.a: " + \iLength)
               AddGadgetItem(miEditorGadgetNumber, -1, "  iDescriptorType.a: " + \iDescriptorType)
               AddGadgetItem(miEditorGadgetNumber, -1, "  iBcdUsb.u: $" + Hex(\iBcdUsb))
               AddGadgetItem(miEditorGadgetNumber, -1, "  iDeviceClass.a: $" + Hex(\iDeviceClass))
               AddGadgetItem(miEditorGadgetNumber, -1, "  iDeviceSubClass.a: $" + Hex(\iDeviceSubClass))
               AddGadgetItem(miEditorGadgetNumber, -1, "  iDeviceProtocol.a: $" + Hex(\iDeviceProtocol))
               AddGadgetItem(miEditorGadgetNumber, -1, "  iMaxPacketSize0.a: " + \iMaxPacketSize0)
               AddGadgetItem(miEditorGadgetNumber, -1, "  iVendorId.u: $" + Hex(\iVendorId))
               AddGadgetItem(miEditorGadgetNumber, -1, "  iProductId.u: $" + Hex(\iProductId))
               AddGadgetItem(miEditorGadgetNumber, -1, "  iBcdDevice.u: $" + Hex(\iBcdDevice))
               AddGadgetItem(miEditorGadgetNumber, -1, "  iManufacturer.a: " + \iManufacturer)
               AddGadgetItem(miEditorGadgetNumber, -1, "  iProduct.a: " + \iProduct)
               AddGadgetItem(miEditorGadgetNumber, -1, "  iSerialNumber.a: " + \iSerialNumber)
               AddGadgetItem(miEditorGadgetNumber, -1, "  iNumConfigurations.a: " + \iNumConfigurations)
            EndWith
         EndIf
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure LibUsbDescriptorLanguageList(*uDeviceHandle)
      DebugView::ProcIn()
      Protected c.i, sError.String
      NewList LanguageIdList.i();

      AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
      AddGadgetItem(miEditorGadgetNumber, -1, "Sting Language IDs (Country Code)")
      If *uDeviceHandle
         If (Not LibUsb::GetStringLanguageList(*uDeviceHandle, LanguageIdList(), sError))
            AddGadgetItem(miEditorGadgetNumber, -1, sError\s)

         Else
            ForEach LanguageIdList()
               c + 1
               AddGadgetItem(miEditorGadgetNumber, -1, "  " + Str(c) + ": 0x" + Hex(LanguageIdList()))
            Next
            FreeList(LanguageIdList())
         EndIf
      Else
         AddGadgetItem(miEditorGadgetNumber, -1, "  *uDeviceHandle ist #Null")
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure LibUsbDescriptorStringAscii(*uDeviceHandle)
      DebugView::ProcIn()
      Protected sString.String, sError.String
      Protected *uDevice.LibUsb::uDevice, uDeviceDescriptor.LibUsb::uDeviceDescriptor

      AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
      AddGadgetItem(miEditorGadgetNumber, -1, "DeviceDescriptor Strings Ascii")
      If *uDeviceHandle
         If (Not LibUsb::GetDevice(*uDeviceHandle, @*uDevice, sError))
            AddGadgetItem(miEditorGadgetNumber, -1, sError\s)

         Else
            If LibUsb::GetDeviceDescriptor(*uDevice, @uDeviceDescriptor, sError)
               With uDeviceDescriptor
                  sString\s = #Null$
                  If \iManufacturer
                     LibUsb::GetStringDescriptorAscii(*uDeviceHandle, \iManufacturer, sString, sError)
                  EndIf
                  AddGadgetItem(miEditorGadgetNumber, -1, "  iManufacturer: " + sString\s)
                  sString\s = #Null$
                  If \iProduct
                     LibUsb::GetStringDescriptorAscii(*uDeviceHandle, \iProduct, sString, sError)
                  EndIf
                  AddGadgetItem(miEditorGadgetNumber, -1, "  iProduct: " + sString\s)
                  sString\s = #Null$
                  If \iSerialNumber
                     LibUsb::GetStringDescriptorAscii(*uDeviceHandle, \iSerialNumber, sString, sError)
                  EndIf
                  AddGadgetItem(miEditorGadgetNumber, -1, "  iSerialNumber: " + sString\s)
               EndWith
            Else
               AddGadgetItem(miEditorGadgetNumber, -1, "  Fehler bei LibUsb::GetDeviceDescriptor: " + sError\s)
            EndIf
         EndIf
      Else
         AddGadgetItem(miEditorGadgetNumber, -1, "  *uDevice oder *uDeviceHandle ist #Null")
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure LibUsbDescriptorStringUnicode(*uDeviceHandle)
      DebugView::ProcIn()
      Protected sString.String, sError.String
      Protected *uDevice.LibUsb::uDevice, uDeviceDescriptor.LibUsb::uDeviceDescriptor
      ;Protected iLanguageId.i = 0      ; Erste verfügbare Sprache verwenden.
      Protected iLanguageId.i = $0409  ; English, United States - muss immer vorhanden sein.
      
      AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
      AddGadgetItem(miEditorGadgetNumber, -1, "DeviceDescriptor Strings Unicode")
      If *uDeviceHandle
         If (Not LibUsb::GetDevice(*uDeviceHandle, @*uDevice, sError))
            AddGadgetItem(miEditorGadgetNumber, -1, sError\s)

         Else
            If LibUsb::GetDeviceDescriptor(*uDevice, @uDeviceDescriptor, sError)
               With uDeviceDescriptor
                  sString\s = #Null$
                  If \iManufacturer
                     LibUsb::GetStringDescriptor(*uDeviceHandle, \iManufacturer, iLanguageId, sString, sError)
                  EndIf
                  AddGadgetItem(miEditorGadgetNumber, -1, "  iManufacturer: " + sString\s)
                  sString\s = #Null$
                  If \iProduct
                     LibUsb::GetStringDescriptor(*uDeviceHandle, \iProduct, iLanguageId, sString, sError)
                  EndIf
                  AddGadgetItem(miEditorGadgetNumber, -1, "  iProduct: " + sString\s)
                  sString\s = #Null$
                  If \iSerialNumber
                     LibUsb::GetStringDescriptor(*uDeviceHandle, \iSerialNumber, iLanguageId, sString, sError)
                  EndIf
                  AddGadgetItem(miEditorGadgetNumber, -1, "  iSerialNumber: " + sString\s)
               EndWith
            Else
               AddGadgetItem(miEditorGadgetNumber, -1, "  Fehler bei LibUsb::GetDeviceDescriptor: " + sError\s)
            EndIf
         EndIf
      Else
         AddGadgetItem(miEditorGadgetNumber, -1, "  *uDevice oder *uDeviceHandle ist #Null")
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure LibUsbSetConfiguration(*uDeviceHandle)
      DebugView::ProcIn()
      Protected sError.String

      AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
      AddGadgetItem(miEditorGadgetNumber, -1, "SetConfiguration Index 1")
      If *uDeviceHandle
         If (Not LibUsb::SetConfiguration(*uDeviceHandle, 1, sError))
            AddGadgetItem(miEditorGadgetNumber, -1, "  " + sError\s)
         EndIf
      Else
         AddGadgetItem(miEditorGadgetNumber, -1, "  *uDeviceHandle ist #Null")
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure LibUsbGetActiveConfigDescriptor(*uDevice, *uConfigDescriptor.Integer)
      DebugView::ProcIn()
      Protected *uConfigDescriptorLocal.LibUsb::uConfigDescriptor, sError.String

      AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
      AddGadgetItem(miEditorGadgetNumber, -1, "GetActiveConfigDescriptor")
      If *uDevice
         If (Not LibUsb::GetActiveConfigDescriptor(*uDevice, *uConfigDescriptor, sError))
            AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
         Else
            *uConfigDescriptorLocal = *uConfigDescriptor\i
            With *uConfigDescriptorLocal
               AddGadgetItem(miEditorGadgetNumber, -1, "  iLength.a: " + \iLength)
               AddGadgetItem(miEditorGadgetNumber, -1, "  iDescriptorType.a: $" + Hex(\iDescriptorType))
               AddGadgetItem(miEditorGadgetNumber, -1, "  iTotalLength.u: " + \iTotalLength)
               AddGadgetItem(miEditorGadgetNumber, -1, "  iNumInterfaces.a: " + \iNumInterfaces)
               AddGadgetItem(miEditorGadgetNumber, -1, "  iConfigurationValue.a: " + \iConfigurationValue)
               AddGadgetItem(miEditorGadgetNumber, -1, "  iConfiguration.a: " + \iConfiguration)
               AddGadgetItem(miEditorGadgetNumber, -1, "  iAttributes.a: " + \iAttributes)
               AddGadgetItem(miEditorGadgetNumber, -1, "  iMaxPower.a: " + \iMaxPower)
               AddGadgetItem(miEditorGadgetNumber, -1, "  *uInterface.uInterface: $" + Hex(\uInterface))
               AddGadgetItem(miEditorGadgetNumber, -1, "  *aExtra.Ascii: $" + Hex(\aExtra))
               AddGadgetItem(miEditorGadgetNumber, -1, "  iExtraLength.i: " + \iExtraLength)
            EndWith
         EndIf
      Else
         AddGadgetItem(miEditorGadgetNumber, -1, "  *uDevice ist #Null")
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure LibUsbInterface(*uDeviceHandle, *uConfigDescriptor.LibUsb::uConfigDescriptor)
      DebugView::ProcIn()
      Protected c.i, k.i, sString.String, sError.String
      Protected iNumAltSettings.i
      Protected *uInterface.LibUsb::uInterface
      Protected *uInterfaceDescriptors
      Protected *uInterfaceDescriptor.LibUsb::uInterfaceDescriptor


      If *uConfigDescriptor
         For c = 1 To *uConfigDescriptor\iNumInterfaces
            *uInterface = (*uConfigDescriptor\uInterface + ((c - 1) * SizeOf(LibUsb::uInterface)))
            AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
            AddGadgetItem(miEditorGadgetNumber, -1, "Interface " + c)
            AddGadgetItem(miEditorGadgetNumber, -1, "  *uInterface: $" + Hex(*uInterface))

            If *uInterface
               iNumAltSettings = *uInterface\iNumAltSetting
               *uInterfaceDescriptors = *uInterface\uAltSetting
               AddGadgetItem(miEditorGadgetNumber, -1, "  iNumAltSettings: " + iNumAltSettings)
               AddGadgetItem(miEditorGadgetNumber, -1, "  *uInterfaceDescriptors: $" + Hex(*uInterfaceDescriptors))

               If *uInterfaceDescriptors
                  With *uInterfaceDescriptor
                     For k = 1 To iNumAltSettings
                        AddGadgetItem(miEditorGadgetNumber, -1, "")
                        AddGadgetItem(miEditorGadgetNumber, -1, "  InterfaceDescriptor " + k)
                        *uInterfaceDescriptor = (*uInterfaceDescriptors + ((k - 1) * SizeOf(LibUsb::uInterfaceDescriptor)))
                        AddGadgetItem(miEditorGadgetNumber, -1, "    *uInterfaceDescriptor: $" + Hex(*uInterfaceDescriptor))

                        If *uInterfaceDescriptor
                           AddGadgetItem(miEditorGadgetNumber, -1, "    iLength.a: " + \iLength)
                           AddGadgetItem(miEditorGadgetNumber, -1, "    iDescriptorType.a: $" + Hex(\iDescriptorType))
                           AddGadgetItem(miEditorGadgetNumber, -1, "    iInterfaceNumber.a: " + \iInterfaceNumber)
                           AddGadgetItem(miEditorGadgetNumber, -1, "    iAlternateSetting.a: " + \iAlternateSetting)
                           AddGadgetItem(miEditorGadgetNumber, -1, "    iNumEndpoints.a: " + \iNumEndpoints)
                           AddGadgetItem(miEditorGadgetNumber, -1, "    iInterfaceClass.a: $" + Hex(\iInterfaceClass))
                           AddGadgetItem(miEditorGadgetNumber, -1, "    iInterfaceSubClass.a: $" + Hex(\iInterfaceSubClass))
                           AddGadgetItem(miEditorGadgetNumber, -1, "    iInterfaceProtocol.a: $" + Hex(\iInterfaceProtocol))
                           AddGadgetItem(miEditorGadgetNumber, -1, "    iInterface.a: " + \iInterface)
                           If *uDeviceHandle And \iInterface
                              If LibUsb::GetStringDescriptorAscii(*uDeviceHandle, \iInterface, sString, sError)
                                 AddGadgetItem(miEditorGadgetNumber, -1, "      (iInterface.a String ASCII: " + sString\s + ")")
                              Else
                                 AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
                              EndIf
                           EndIf
                           AddGadgetItem(miEditorGadgetNumber, -1, "    *uEndpoint.uEndpointDescriptor: $" + Hex(\uEndpoint))
                           AddGadgetItem(miEditorGadgetNumber, -1, "    *aExtra.Ascii: $" + Hex(\aExtra))
                           AddGadgetItem(miEditorGadgetNumber, -1, "    iExtraLength.i: " + \iExtraLength)
                        EndIf

                        ; Endpoints ermitteln.
                        LibUsbEndpoint(*uInterfaceDescriptor)
                     Next k
                  EndWith
               EndIf
            EndIf
         Next c
      Else
         AddGadgetItem(miEditorGadgetNumber, -1, "Interface:  *uConfigDescriptor ist #Null")
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure LibUsbEndpoint(*uInterfaceDescriptor.LibUsb::uInterfaceDescriptor)
      DebugView::ProcIn()
      Protected i.i
      Protected *uEndpoints, *uEndpoint.LibUsb::uEndpointDescriptor

      If *uInterfaceDescriptor
         *uEndpoints = *uInterfaceDescriptor\uEndpoint
         If *uEndpoints
            For i = 1 To *uInterfaceDescriptor\iNumEndpoints
               *uEndpoint = (*uEndpoints + ((i - 1) * SizeOf(LibUsb::uEndpointDescriptor)))
               If *uEndpoint
                  With *uEndpoint
                     AddGadgetItem(miEditorGadgetNumber, -1, "")
                     AddGadgetItem(miEditorGadgetNumber, -1, "    Endpoint " + Str(i))
                     AddGadgetItem(miEditorGadgetNumber, -1, "      iLength.a: " + \iLength)
                     AddGadgetItem(miEditorGadgetNumber, -1, "      iDescriptorType.a: $" + Hex(\iDescriptorType))
                     AddGadgetItem(miEditorGadgetNumber, -1, "      iEndpointAddress.a: $" + Hex(\iEndpointAddress))
                     AddGadgetItem(miEditorGadgetNumber, -1, "      iAttributes.a: " + \iAttributes)
                     AddGadgetItem(miEditorGadgetNumber, -1, "      iMaxPacketSize.u: " + \iMaxPacketSize)
                     AddGadgetItem(miEditorGadgetNumber, -1, "      iInterval.a: " + \iInterval)
                     AddGadgetItem(miEditorGadgetNumber, -1, "      iRefresh.a: " + \iRefresh)
                     AddGadgetItem(miEditorGadgetNumber, -1, "      iSynchAddress.a: " + \iSynchAddress)
                     AddGadgetItem(miEditorGadgetNumber, -1, "      *aExtra.Ascii: $" + Hex(\aExtra))
                     AddGadgetItem(miEditorGadgetNumber, -1, "      iExtraLength.i: " + \iExtraLength)
                  EndWith
               EndIf
            Next i
         EndIf
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure LibUsbFreeConfigDescriptor(*uConfigDescriptor)
      DebugView::ProcIn()

      AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
      AddGadgetItem(miEditorGadgetNumber, -1, "FreeConfigDescriptor")
      If *uConfigDescriptor
         LibUsb::FreeConfigDescriptor(*uConfigDescriptor)
      Else
         AddGadgetItem(miEditorGadgetNumber, -1, "  *uConfigDescriptor ist #Null")
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure.i LibUsbClaimInterface(*uDeviceHandle, iInterface.i)
      DebugView::ProcIn()
      Protected fReturn.i, sError.String

      If *uDeviceHandle
         AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
         AddGadgetItem(miEditorGadgetNumber, -1, "ClaimInterface " + iInterface)
         fReturn = LibUsb::ClaimInterface(*uDeviceHandle, iInterface, sError)
         If (Not fReturn)
            AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i LibUsbReleaseInterface(*uDeviceHandle, iInterface.i)
      DebugView::ProcIn()
      Protected sError.String

      If *uDeviceHandle
         AddGadgetItem(miEditorGadgetNumber, -1, #Null$)
         AddGadgetItem(miEditorGadgetNumber, -1, "ReleaseInterface " + iInterface)
         If (Not LibUsb::ReleaseInterface(*uDeviceHandle, iInterface, sError))
            AddGadgetItem(miEditorGadgetNumber, -1, sError\s)
         EndIf
      EndIf

      DebugView::ProcOut()
   EndProcedure

   ;-

   Procedure.i Open()
      ; Das Programmfenster laden und anzeigen.
      DebugView::ProcIn()
      Protected iFontNumber.i

      miWindowNumber = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, #iWindowWidth, #iWindowHeight, #sWindowTitle,
                                  #PB_Window_MinimizeGadget | #PB_Window_MaximizeGadget | #PB_Window_SizeGadget)
      If miWindowNumber
         WindowBounds(miWindowNumber, #iWindowMinWidth, #iWindowMinHeight, #PB_Ignore, #PB_Ignore)

         BindEvent(#PB_Event_CloseWindow, @EventCloseWindow(), miWindowNumber)
         BindEvent(#PB_Event_MinimizeWindow, @EventMinimizeWindow(), miWindowNumber)
         BindEvent(#PB_Event_SizeWindow, @EventSizeWindow(), miWindowNumber)
         BindEvent(#PB_Event_RestoreWindow, @EventRestoreWindow(), miWindowNumber)

         Tool::WindowReduceFlicker(miWindowNumber)

         miMenuNumber = CreateMenu(#PB_Any, WindowID(miWindowNumber))
         If miMenuNumber
            MenuTitle(#sMenuTitleFile)
            MenuItem(#eMenuItemFileExit, #sMenuItemFileExit)
            MenuTitle(#sMenuTitleEdit)
            MenuTitle(#sMenuTitleHelp)
            MenuItem(#eMenuItemHelpLog, #sMenuItemHelpLog)
            MenuItem(#eMenuItemHelpAbout, #sMenuItemHelpAbout)

            BindMenuEvent(miMenuNumber, #eMenuItemFileExit, @EventMenuFileExit())
            BindMenuEvent(miMenuNumber, #eMenuItemHelpLog, @EventMenuHelpLog())
            BindMenuEvent(miMenuNumber, #eMenuItemHelpAbout, @EventMenuHelpAbout())
         EndIf

         iFontNumber = LoadFont(#PB_Any, #sFontName, #iFontSize)

         miListViewGadgetNumber = ListViewGadget(#PB_Any, 10, 10, 780, 210)
         If miListViewGadgetNumber
            SetGadgetFont(miListViewGadgetNumber, FontID(iFontNumber))

            AddGadgetItem(miListViewGadgetNumber, -1,
                          "LibUsb-Version - LibUsb::GetVersion()")
            SetGadgetItemData(miListViewGadgetNumber, CountGadgetItems(miListViewGadgetNumber) - 1, #iListViewGetVersion)
            AddGadgetItem(miListViewGadgetNumber, -1,
                          "Fehlernamen auflisten - LibUsb::ErrorName()")
            SetGadgetItemData(miListViewGadgetNumber, CountGadgetItems(miListViewGadgetNumber) - 1, #iListViewErrorName)
            AddGadgetItem(miListViewGadgetNumber, -1,
                          "Fehlerbeschreibungen auflisten - LibUsb::ErrorDescription() mit SetLocale(), ErrorName(), StrError()")
            SetGadgetItemData(miListViewGadgetNumber, CountGadgetItems(miListViewGadgetNumber) - 1, #iListViewErrorDescription)
            AddGadgetItem(miListViewGadgetNumber, -1,
                          "Alle USB-Geräte auflisten - LibUsbDevices()")
            SetGadgetItemData(miListViewGadgetNumber, CountGadgetItems(miListViewGadgetNumber) - 1, #iListViewDevices)

            BindGadgetEvent(miListViewGadgetNumber, @EventListViewGadgetLeftClick(), #PB_EventType_LeftClick)
         EndIf

         miEditorGadgetNumber = EditorGadget(#PB_Any, 10, 230, 780, 210)
         SetGadgetFont(miEditorGadgetNumber, FontID(iFontNumber))

         miSplitterGadgetNumber = SplitterGadget(#PB_Any, 10, 10, 780, 430,
                                                 miListViewGadgetNumber, miEditorGadgetNumber) ;, #PB_Splitter_Separator)
         SetGadgetState(miSplitterGadgetNumber, 150)

         PostEvent(#PB_Event_SizeWindow, miWindowNumber, #Null)
      Else
         DebugView::Error("Das Fenster >" + #PB_Compiler_Module + "< konnte nicht erstellt werden.")
      EndIf

      DebugView::ProcOut()
      ProcedureReturn miWindowNumber
   EndProcedure


   Procedure Close()
      ; Schließt das Fenster und Programm.
      DebugView::ProcIn()

      If miWindowNumber
         UnbindEvent(#PB_Event_CloseWindow, @EventCloseWindow(), miWindowNumber)
         UnbindEvent(#PB_Event_MinimizeWindow, @EventMinimizeWindow(), miWindowNumber)
         UnbindEvent(#PB_Event_SizeWindow, @EventSizeWindow(), miWindowNumber)
         UnbindEvent(#PB_Event_RestoreWindow, @EventRestoreWindow(), miWindowNumber)

         UnbindMenuEvent(miMenuNumber, #eMenuItemFileExit, @EventMenuFileExit())
         UnbindMenuEvent(miMenuNumber, #eMenuItemHelpLog, @EventMenuHelpLog())
         UnbindMenuEvent(miMenuNumber, #eMenuItemHelpAbout, @EventMenuHelpAbout())

         UnbindGadgetEvent(miListViewGadgetNumber, @EventListViewGadgetLeftClick(), #PB_EventType_LeftClick)

         CloseWindow(miWindowNumber)
         miWindowNumber = #Null

         ; Ereignis für das Beenden des Programms an die Hauptschleife senden.
         PostEvent(#eCustomEventExitApplication)

      EndIf

      DebugView::ProcOut()
      ProcedureReturn
   EndProcedure


EndModule   ; WindowStart.
